/*
 * Title:        bits/map_helper.hpp
 * Description:  C++ header-only wrapper for C-ABI data_t type
 * $Copyright (c) 2015-2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 * @Version:     $Id: map_helper.hpp 339879 2018-10-25 07:42:54Z antm $
 */

/* This file:
 * Contains the C++ implementation of MapHelper
 *
 */

#ifndef _DATAT_BITS_INCLUDE_TAG
#error Must only be included from sag_connectivity_cpp.hpp, use that instead
#endif

/**
 * [DEPRECATED]\n
 * Helper class for extracting values from a string->data_t map_t in a type-safe way with 
 * error checking and support for default values.
 *
 * It can be used by plugins for extracting values from message payloads.
 *
 * @deprecated This class has been deprecated; we recommend using the MapExtractor class for this purpose instead.
 *
 */
class MapHelper
{
public:
	// String
	/** Get a string value from the map, throwing if the key is not present. */
	static std::string getString(const map_t &map, const char *key) { return getAs<std::string>(map, key);	}
	/** Get a string value from the map, returning the given default if the key is not present. */
	static std::string getString(const map_t &map, const char *key, const char *defaultValue) { return getAs<std::string>(map, key, defaultValue);	}
	/** Get a string value from the map, throwing if the key is not present. */
	static std::string getString(const map_t &map, const std::string &key) { return getAs<std::string>(map, key.c_str());	}
	/** Get a string value from the map, returning the given default if the key is not present. */
	static std::string getString(const map_t &map, const std::string &key, const std::string &defaultValue) { return getAs<std::string>(map, key.c_str(), defaultValue);	}
	// Integer
	/** Get an integer value from the map, throwing if the key is not present. */
	static int64_t getInteger(const map_t &map, const char *key) { return getAs<int64_t>(map, key);	}
	/** Get an integer value from the map, returning the given default if the key is not present. */
	static int64_t getInteger(const map_t &map, const char *key, int64_t defaultValue) { return getAs<int64_t>(map, key, defaultValue);	}
	/** Get an integer value from the map, throwing if the key is not present. */
	static int64_t getInteger(const map_t &map, const std::string &key) { return getAs<int64_t>(map, key.c_str());	}
	/** Get an integer value from the map, returning the given default if the key is not present. */
	static int64_t getInteger(const map_t &map, const std::string &key, int64_t defaultValue) { return getAs<int64_t>(map, key.c_str(), defaultValue);	}
	// Double
	/** Get a double value from the map, throwing if the key is not present. */
	static double getDouble(const map_t &map, const char *key) { return getAs<double>(map, key);	}
	/** Get a double value from the map, returning the given default if the key is not present. */
	static double getDouble(const map_t &map, const char *key, double defaultValue) { return getAs<double>(map, key, defaultValue);	}
	/** Get a double value from the map, throwing if the key is not present. */
	static double getDouble(const map_t &map, const std::string &key) { return getAs<double>(map, key.c_str());	}
	/** Get a double value from the map, returning the given default if the key is not present. */
	static double getDouble(const map_t &map, const std::string &key, double defaultValue) { return getAs<double>(map, key.c_str(), defaultValue);	}
	// Boolean
	/** Get a boolean value from the map, throwing if the key is not present. */
	static bool getBoolean(const map_t &map, const char *key) { return getAs<bool>(map, key);	}
	/** Get a boolean value from the map, returning the given default if the key is not present. */
	static bool getBoolean(const map_t &map, const char *key, bool defaultValue) { return getAs<bool>(map, key, defaultValue);	}
	/** Get a boolean value from the map, throwing if the key is not present. */
	static bool getBoolean(const map_t &map, const std::string &key) { return getAs<bool>(map, key.c_str());	}
	/** Get a boolean value from the map, returning the given default if the key is not present. */
	static bool getBoolean(const map_t &map, const std::string &key, bool defaultValue) { return getAs<bool>(map, key.c_str(), defaultValue);	}
	// Map
	/** Get a map value from the map, throwing if the key is not present. */
	static const map_t &getMap(const map_t &map, const char *key) { return getAs<const map_t&>(map, key);	}
	/** Get a map value from the map, returning the given default if the key is not present. */
	static const map_t &getMap(const map_t &map, const char *key, const map_t &defaultValue) { return getAs<const map_t&>(map, key, defaultValue);	}
	/** Get a map value from the map, throwing if the key is not present. */
	static const map_t &getMap(const map_t &map, const std::string &key) { return getAs<const map_t&>(map, key.c_str());	}
	/** Get a map value from the map, returning the given default if the key is not present. */
	static const map_t &getMap(const map_t &map, const std::string &key, const map_t &defaultValue) { return getAs<const map_t&>(map, key.c_str(), defaultValue);	}
	// List
	/** Get a list value from the map, throwing if the key is not present. */
	static const list_t &getList(const map_t &map, const char *key) { return getAs<const list_t&>(map, key);	}
	/** Get a list value from the map, returning the given default if the key is not present. */
	static const list_t &getList(const map_t &map, const char *key, const list_t &defaultValue) { return getAs<const list_t&>(map, key, defaultValue);	}
	/** Get a list value from the map, throwing if the key is not present. */
	static const list_t &getList(const map_t &map, const std::string &key) { return getAs<const list_t&>(map, key.c_str());	}
	/** Get a list value from the map, returning the given default if the key is not present. */
	static const list_t &getList(const map_t &map, const std::string &key, const list_t &defaultValue) { return getAs<const list_t&>(map, key.c_str(), defaultValue);	}

private:
	/** @internal Templated getter */
	template<typename T>
	static T getAs(const map_t &map, const char *key);
	/** @internal Templated getter with default */
	template<typename T>
	static T getAs(const map_t &map, const char *key, const T &def);
};


